clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% V Y T V O R E N I    V S T U P N I C H    D A T    
% (T E O R E T I C K A    S I M U L A C E)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ts = 0.2;                   % perioda vzorkovani
u = 0.1;                    
noise = 0.001;                % velikost sumu PSD
time = 20;                  % delka simulace
p = tf('p');                % laplaceuv operator
Gs = 2/(5*p+1);            % identifikovany system
Gz = c2d(Gs,Ts);            % diskretizace systemu
[num,den] = tfdata(Gz,'v'); % uprava dat pro simulaci
num(2) = 0.5;
den(2) = -0.5;
sim('model.mdl',time)       % spusteni simulace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% U P R A V A    V S T U P N I C H    D A T
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m,n] = size(Data);         % m...radky, n...sloupce
% fi...matice vstupnich a vystupnich dat
uk1 = Data(1:m-1,3);   % fi(:,1) u(k-1)
yk1 = Data(1:m-1,2);   %fi(:,2) y
Y = Data(2:m,2);            % Y...vektor vzoru
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% A R X    M O D E L
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
iterm=300;
theta = rand(2,1)*2-1;             %hledane parametry
J = -[uk1 -yk1];
for iter=1:iterm
    E = Y - (uk1*theta(1)-yk1*theta(2));  %chyba  
    g = J'*E;   %gradient 
    H = (J'*J + 0.5*[1 0;0 1]);
    p=H\g;
    if (E~=0)
        eta = fminsearchbnd(@(eta) fetaminOE1r(eta,uk1,yk1,Y,theta,p),0,0,1,optimset('MaxIter',20));
        theta = theta - eta*p;
    end 
    Iarx(iter) = E'*E;
    w(:,iter) = theta; 
end
disp('ARX')
theta

[b1 a1] = meshgrid(0:0.1:1,-1:0.1:0);
[mm,nn] = size(b1);
for im = 1:mm
    for in = 1:mm
        Ym = uk1.*b1(im,in) - yk1.*a1(im,in);
        II(im,in) = sum((Y - Ym).^2);
    end
end
figure(1)
meshc(b1,a1,II)
hold on
plot3(w(1,iterm),w(2,iterm),Iarx(iterm),'kx')
title('Pozice minima kriterialni funkce pro ARX model')
xlabel('b1')
ylabel('a1')

figure(2)
plot(1:m-1,Y)
hold on
ymk1 = zeros(m-1,1);
for i=1:m-1
    Yarx(i) =  uk1(i)*theta(1)-ymk1(i)*theta(2);
    if (i~=m-1) ymk1(i+1) = Yarx(i); end;
end
plot(1:m-1,Yarx,'r')
Y1kp = uk1*theta(1)-yk1*theta(2);
plot(1:m-1,Y1kp,'k')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% O E    M O D E L
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m_initial = idpoly(1,[0 theta(1)],1,1,[1 theta(2)],1,Ts);
data = iddata(Data(1:m,2),Data(1:m,3),Ts);
m1 = oe(data,m_initial)
m2 = oe(data,[1 1 1])

% theta = [1 0]';
for iter=1:iterm
    %-------------------vypocet predikci-----------------------------------
    ymk1 = zeros(m-1,1);
    for i=1:m-1
        Yoe(i,1) =  uk1(i)*theta(1)-ymk1(i)*theta(2);
        if (i~=m-1) ymk1(i+1) = Yoe(i); end;
    end
    %----------------------------------------------------------------------
    E = Y - Yoe;
    %--------------------vypocet jacobianu---------------------------------
    uJ = OEfiltr(uk1,theta(2));
    yJ = OEfiltr(ymk1,theta(2));
    J(:,1) = -uJ;   
    J(:,2) = -yJ; 
%     J(1,:) = -[uk1(1), -ymk1(1)];
%     for i=2:m-1
%         J(i,:) = -[uk1(i-1)-theta(2)*J(i-1,1), -ymk1(i-1)-theta(2)*J(i-1,2)];
%     end
    %----------------------------------------------------------------------
    g = J'*E;
    H = (J'*J + 0.5*[1 0;0 1]);
    p=H\g;
    if (E~=0)
        eta = fminsearchbnd(@(eta) fetaminOE1r(eta,uk1,ymk1,Y,theta,g),0,0,1,optimset('MaxIter',20));
%         eta = 0.002;
        theta = theta - eta*p
    end
    Ioe(iter) = E'*E;
    w(:,iter) = theta; 
end
disp('OE')
theta
plot(1:m-1,Yoe,'g')
title('Porovnani ARX a OE')
xlabel('krok')
ylabel('y')
legend('Soustava','ARX xk predikce','ARX 1k predikce','OE xk predikce')

for im = 1:mm
    for in = 1:mm
        Y2 = uk1.*b1(im,in) - ymk1.*a1(im,in);
        III(im,in) = sum((Y - Y2).^2);
    end
end
figure(3)
meshc(b1,a1,III)
hold on
plot3(w(1,iterm),w(2,iterm),Ioe(iterm),'kx')
title('Pozice minima kriterialni funkce pro OE model')
xlabel('b1')
ylabel('a1')
